/*{ Template_FreeRTOS::main.c ********************************************
 
 * (w) 20100719 by Gregor Rebel
 
 * Basic framework template of blinking LED for
     - Olimex STM32-P103 eval board
     - StdPeripheralsLibrary CMSIS V3.4.0 (tested)
     - GNU toolchain from CodeSourcery G++
     - local/ remote debugging (JTAG-Adapter may be connected to different computer)


 * Coding-style
    + gnu99
      It's year 2011, get a decent compiler if your's does not comply with this.
      arm-none-eabi-gcc-4.4.1 does support gnu99.
      
    + Text Folding
      This source makes use of text folding to improve hierarchical readability 
      It has been written with jEdit + plugin Configurable Fold Handler.
      Activate folding in your favourite text-editor for { and } 
      (fold-start and -end) for best visual experience.


 * Compile:
   ./compile.sh
   or
   make all

 * Flash onto u-controller:
   - change _openocd_flash.sh according to your board and adapter
   - flash binary
     ./_/openocd_flash.sh main.bin
   
 * Debug:
   - change openocd_gdb.sh and gdb.sh according to your board and adapter
   - start openocd as gdb-server (may even run on another computer connected over network)
     ./_/openocd_gdb.sh
   - start gdb text user-interface in another shell
     ./_/gdb.sh
     
         

 * Configuration of source-code:
    + stm32f10x_conf.h
      - modify according to your hardware (if differing from above) 
    
    + stm32f10x_it.c .
      - implement your own interrupthandlers in existing functions
      
    + main.c (this file)
      - extend as you like
      
    + makefile
      - all .c, .h and .s files in local directory are compiled and linked together automatically.
      
      
 * Prerequisites:
   + OpenOCD
     - configure & run  install/install_OpenOCD.sh
     
   + CodeSourcery G++
     - configure & run  install/install_SourceryG++.sh
     - add to PATH (replace <XXX>according to your local filesystem):
       export PATH="<XXX>/SourceryG++/arm-2010q1/bin/:<XXX>/SourceryG++/arm-2010q1/libexec/gcc/arm-none-eabi/4.4.1/:$PATH"
 
   + StdPeripheralsLibrary CMSIS V3.3.0
     - configure & run  install/install_STM-StdPeripheralsLibrary.sh
     - create link named StdPeripheralsLibrary to subfolder Libraries/ of unpacked archive 

    cp StdPeripheralsLibrary/CMSIS/CM3/DeviceSupport/ST/STM32F10x/stm32f10x.h Libraries/CMSIS/CM3/DeviceSupport/ST/STM32F10x/stm32f10x_orig.h
    chmod +w Libraries/CMSIS/CM3/DeviceSupport/ST/STM32F10x/stm32f10x.h
       - move assert-macro from stm32f10x_conf.h into stm32f10x.h
       - source: http://www.mikrocontroller.net/topic/156063#1472643

    cp StdPeripheralsLibrary/CMSIS/CM3/DeviceSupport/ST/STM32F10x/startup/gcc_ride7/startup_stm32f10x_md.s Libraries/CMSIS/CM3/DeviceSupport/ST/STM32F10x/startup/gcc_ride7/startup_stm32f10x_md_orig.s
    chmod +w Libraries/CMSIS/CM3/DeviceSupport/ST/STM32F10x/startup/gcc_ride7/startup_stm32f10x_md.s
      - change line (ResetHandler Bug: http://www.mikrocontroller.net/topic/156063#1476546):
        .word	Reset_Handler
     -> .word	Reset_Handler + 1
   
  * Scripts:
    Some helper scripts can be found inside scripts/.
    Each script-name starts with a comma to make it easy to tab them after adding scripts/ to $PATH.
    
    clean.sh           removes all object files in current and all subfolders
    compile.sh         cleans, compiles and flashes application 
    cscope.sh          creates a searchable index of all .c and .h files (jEdit provides cscope-plugin)
    findInLibrary.sh   searches for given text in all .h files inside StdPeripheralsLibrary
    findInSources.sh   searches for given text in all .h files of given folder
    flash.sh           kills running openocd + starts openocd_flash.sh
    gdb.sh             starts remote gdb-session (change according to your setup)
    jedit.sh           wrapper for cscope to use jEdit as Source-Browser (download from jedit.org)
                      Issue before starting cscope: export CSCOPE_EDITOR=`pwd`/scripts/jedit.sh
    openocd_flash.sh   starts openocd to flash given binary onto uC 
    openocd_gdb.sh     starts openocd as GDB server waiting for a GDB to connect
    paths.sh           can be sourced to provide all required paths (adapt to your setting before)
                       Usage: add a line similar to this to your ~/.bashrc
                       source $HOME/Source/STM32_Template/scripts/paths.sh

}*/
//{ Defines/ TypDefs *****************************************************


//} Defines
//{ Includes *************************************************************

/**
 * IO definitions
 *
 * define access restrictions to peripheral registers
 *
 * (Copied from core_cm3.h because its required to include stm32f10x.h which is required t include core_cm3.h!)
 */

 // Basic set of helper functions
#include "stm32_io.h"

#include <stdlib.h>

#define USE_STDPERIPH_DRIVER
#include "stm32f10x.h"
#include "core_cm3.h"      // StdPeripheralsLibrary/CMSIS/CM3/CoreSupport/
#include "stm32f10x_conf.h"

// ADC
#include "stm32f10x_adc.h"
#include "stm32f10x_dma.h"

// Timers
#include "stm32f10x_tim.h"

#include "stm32f10x_it.h"
#include "system_stm32f10x.h"
#include "bits.h"

#include <string.h>
#include "FreeRTOS.h"
#include "task.h"

#ifdef EXAMPLE_LEDS
#include "example_leds.h"
#endif
#ifdef EXAMPLE_LCD_320x240
#include "example_lcd_320x240.h"
#endif
//} Includes
//{ Global Variables *****************************************************



//} Global Variables
//{ Function prototypes **************************************************

int main(void);

// task which spawns all other tasks + runs main eventloop
void taskMain(void *pvParameters);

// called in case pvPortMalloc() fails
void vApplicationMallocFailedHook(void);

//} Function prototypes
//{ Function definitions *************************************************

void vApplicationMallocFailedHook(void) {
    while (1);
}
        int main(void) {

    SystemInit();
    
    xTaskCreate(taskMain,                          // function to start as thread
                (const signed char*) "taskMain",   // thread name (just for debugging)
                (size_t) 512, // stack size
                (void *) 0,                        // passed as argument to taskLEDs()                       
                (unsigned portBASE_TYPE) 1,        // task priority (higher values mean more process time)
                (xTaskHandle*) NULL                // can return a handle to created task
               );
    
    vTaskStartScheduler(); // FreeRTOS-scheduler (never returns)
}
       void taskMain(void *pvParameters) {

#ifdef EXAMPLE_LEDS
    leds_start();
#endif

#ifdef EXAMPLE_LCD_320x240
    lcd_start();
#endif

while (1);
}

//} Function definitions
